# @(#) $Id: dirindex.tcl,v 1.3 2000/01/28 07:17:48 mayoff Exp $

# Initialization

proc dirindex_initialize {} {
    set mimeIcons [ns_configsection ns/mimeicons]
    set size [ns_set size $mimeIcons]
    ns_share dirindex_mimeImgTag
    for {set i 0} {$i < $size} {incr i} {
	set mimeType [ns_set key $mimeIcons $i]
	set imageFile [ns_set value $mimeIcons $i]
	if [catch { set imageSize [ns_gifsize [ns_url2file $imageFile]] } error] {
	    ns_log Warning "'$imageFile' icon for '$mimeType': $error"
	} else {
	    set dirindex_mimeImgTag($mimeType) "<img alt='$mimeType' width='[lindex $imageSize 0]' height='[lindex $imageSize 1]' src='$imageFile'>"
	}
    }
}

dirindex_initialize

proc safeGet {varName} {
    upvar $varName var
    if {[info exists var]} {
	return $var
    }

    return ""
}

proc dirindex_getImgTag {type} {
    ns_share dirindex_mimeImgTag

    set tag [safeGet dirindex_mimeImgTag($type)]
    if {$tag != ""} { return $tag }

    set i [string first / $type]
    if {$i >= 0} {
	set type "[string range $type 0 $i]*"
	set tag [safeGet dirindex_mimeImgTag($type)]
	if {$tag != ""} { return $tag }
    }

    return [safeGet dirindex_mimeImgTag(unknown)]
}

proc dirindex_humanReadableSize {bytes} {
    if {$bytes > 1073741824} {

	set size [expr round($bytes / 1073741824.0)]
	return [format "%dG" $size]

    } elseif {$bytes > 1048576} {

	set size [expr round($bytes / 1048576.0)]
	return [format "%dM" $size]

    } elseif {$bytes >= 1024} {

	set size [expr round($bytes / 1024.0)]
	return [format "%dK" $size]

    } else {
	return $bytes
    }
}

proc dirindex_compare_name {a b} {
	upvar name field
	return [string compare $field($a) $field($b)]
}

proc dirindex_compare_type {a b} {
	upvar type field
	return [string compare $field($a) $field($b)]
}

proc dirindex_compare_size {a b} {
	upvar size field
	return [expr $field($a) - $field($b)]
}

proc dirindex_compare_mtime {a b} {
	upvar mtime field
	return [expr $field($a) - $field($b)]
}

proc dirindex {} {

    set url [ns_conn url]

    if {[string range $url end end] != "/"} {
	ns_returnredirect "[ns_conn location]$url/"
	return
    }

    set dir [ns_url2file $url]
    set form [ns_getform]
    set query [ns_conn query]
    if {$query != ""} {
	set query "?$query"
    }

    set sortableFields {name mtime size type}

    set dirLength [string length $dir]
    set fileNameList [glob -nocomplain -- "$dir*"]
    set indexes {}

    set i 0
    foreach file $fileNameList {

	lappend indexes $i

	set name($i) [string range $file $dirLength end]

	file stat $file stat
	set mtime($i) $stat(mtime)
	set size($i) $stat(size)

	if { $stat(type) == "directory" } {
	    set type($i) dir
	    set name($i) "$name($i)/"
	} else {
	    set type($i) [ns_guesstype $file]
	    if { $type($i) == "" } {
		set type($i) unknown
	    }
	}

	set imgTag($i) [dirindex_getImgTag $type($i)]

	incr i
    }

    if {$form == ""} {
	set sortField name
	set sortIndex 0
	set sortOrder -increasing
	set sameOrder a
	set oppositeOrder d
    } else {
	set sortField [ns_set get $form sort]
	set sortIndex [lsearch -exact $sortableFields $sortField]
	if { $sortIndex == -1 } {
	    set sortIndex 0
	    set sortField [lindex $sortableFields $sortIndex]
	}

	if {[string compare [ns_set get $form order] d] == 0} {
	    set sortOrder "-decreasing"
	    set sameOrder d
	    set oppositeOrder a
	} else {
	    set sortOrder "-increasing"
	    set sameOrder a
	    set oppositeOrder d
	}
    }

    foreach field $sortableFields {
	set fieldATag($field) "<a href='[ns_conn url]?sort=$field&amp;order=[
	    expr {$field == $sortField ? $oppositeOrder : $sameOrder}]'>"
    }

    set indexes [lsort \
	$sortOrder \
	-command "dirindex_compare_$sortField" \
	$indexes
    ]

    set result \
"<html>
    <head>
	<title>Contents of [ns_quotehtml $url]</title>
    </head>
    <body>
	<table border='0' cellspacing='0'>
	    <tr><th align='center' colspan='9'>Contents of [ns_quotehtml $url]</td></tr>
	    <tr bgcolor='#dddddd'>
		<td align='center' colspan='9'><a href='../$query'>Go up one level</a></td>
	    <tr bgcolor='#dddddd'>
		<th align='center'>&nbsp;</th>
		<th>&nbsp;</th>
		<th align='left'>$fieldATag(name)Name</a></th>
		<th>&nbsp;</th>
		<th align='left'>$fieldATag(mtime)Last Modified</a></th>
		<th>&nbsp;</th>
		<th align='left'>$fieldATag(size)Size</a></th>
		<th>&nbsp;</th>
		<th align='left'>$fieldATag(type)Type</a></th>
	    </tr>
	    </tr>
"

    set bgcolorList {#eeeeee #eeeeee #eeeeee #ffffff #ffffff #ffffff}
    set bgcolorCount [llength $bgcolorList]

    set bgcolorIndex 0
    foreach i $indexes {
	set htmlname [ns_quotehtml $name($i)]
	append result {<tr bgcolor="}
	append result [lindex $bgcolorList $bgcolorIndex]
	append result {"><td align="center">}
	append result $imgTag($i)
	append result {</td><td>&nbsp;</td><td>}
	append result "<a href='$htmlname$query'>$htmlname</a>"
	append result {</td><td>&nbsp;</td><td>}
	append result [clock format $mtime($i) -format {%Y/%m/%d}]
	append result {</td><td>&nbsp;</td><td align="right">}
	append result [dirindex_humanReadableSize $size($i)]
	append result {</td><td>&nbsp;</td><td align="left">}
	append result $type($i)
	append result "</td></tr>\n"

	incr bgcolorIndex
	if {$bgcolorIndex == $bgcolorCount} { set bgcolorIndex 0 }
    }

    append result \
"	</table>
    </body>
</html>
"

    ns_return 200 "text/html" $result
}

